==> grizzly-1.0.0.tar.gz: lib/src/num/num_mat_iterable.dart <==
import 'package:grizzly/grizzly.dart';
import 'package:grizzly/src/utils/comap.dart';

typedef Num2DView = Iterable<Iterable<num>>;

extension NullableNumMatIterableExt on Iterable<Iterable<num?>> {
  Iterable<num?> get median => map((e) => e.median);

  Iterable<double> get sum => map((e) => e.sum);

  Iterable<double> get mean => map((e) => e.mean);

  Iterable<double> get variance => map((e) => e.variance);

  Iterable<double> get std => map((e) => e.std);
}

extension NumMatIterableExt on Iterable<Iterable<num>> {
  Iterable<Iterable<num>> operator +(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row + other);
    } else if (other is Iterable<num>) {
      return map((e) => e + other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! + row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<num>> operator -(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row - other);
    } else if (other is Iterable<num>) {
      return map((e) => e - other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! - row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<num>> operator *(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row * other);
    } else if (other is Iterable<num>) {
      return map((e) => e * other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! * row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<num>> operator /(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row / other);
    } else if (other is Iterable<num>) {
      return map((e) => e / other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! / row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<int>> operator ~/(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row ~/ other);
    } else if (other is Iterable<num>) {
      return map((e) => e ~/ other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! ~/ row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<num> dot([Iterable<Iterable<num>>? other]) =>
      comapEq<num, Iterable<num>, Iterable<num>>(
          this, other ?? this, (Iterable<num> a, Iterable<num> b) => a.dot(b));

  List<List<double>> toDouble() => map((e) => e.toDouble()).toList();
  List<List<int>> toInt() => map((e) => e.toInt()).toList();
}

==> grizzly-1.0.1.tar.gz: lib/src/num/num_mat_iterable.dart <==
import 'dart:math' as math;
import 'package:grizzly/grizzly.dart';
import 'package:grizzly/src/utils/comap.dart';
import 'package:grizzly/src/utils/reducers.dart';

typedef Num2DView = Iterable<Iterable<num>>;

extension NullableNumMatIterableExt on Iterable<Iterable<num?>> {
  Iterable<num?> get medianRows => map((e) => e.median);

  Iterable<num> get sumRows => map((e) => e.sum);

  Iterable<double> get meanRows => map((e) => e.mean);

  Iterable<double> get varianceRows => map((e) => e.variance);

  Iterable<double> get stdRows => map((e) => e.std);

  num get sum {
    num ret = 0;
    for (final row in this) {
      ret += row.sum;
    }
    return ret;
  }

  double get mean {
    if (length == 0) return 0.0;

    final reducer = MeanReducer();
    for(final row in this) {
      for(var item in row) {
        reducer.consume(item ?? 0);
      }
    }
    return reducer.mean;
  }

  double get variance {
    if (length == 0) return 0.0;

    final reducer = VarianceReducer();
    for(final row in this) {
      for(var item in row) {
        reducer.consume(item ?? 0);
      }
    }
    return reducer.variance;
  }

  double get std => math.sqrt(variance);
}

extension NumMatIterableExt on Iterable<Iterable<num>> {
  Iterable<Iterable<num>> operator +(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row + other);
    } else if (other is Iterable<num>) {
      return map((e) => e + other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! + row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<num>> operator -(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row - other);
    } else if (other is Iterable<num>) {
      return map((e) => e - other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! - row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<num>> operator *(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row * other);
    } else if (other is Iterable<num>) {
      return map((e) => e * other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! * row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<num>> operator /(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row / other);
    } else if (other is Iterable<num>) {
      return map((e) => e / other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! / row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<int>> operator ~/(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row ~/ other);
    } else if (other is Iterable<num>) {
      return map((e) => e ~/ other);
    } else if (other is Iterable<Iterable<num>>) {
      return comap(this, other,
          (Iterable<num>? row1, Iterable<num>? row2) => row1! ~/ row2!);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<num> dot([Iterable<Iterable<num>>? other]) =>
      comapEq<num, Iterable<num>, Iterable<num>>(
          this, other ?? this, (Iterable<num> a, Iterable<num> b) => a.dot(b));

  List<List<double>> toDouble() => map((e) => e.toDouble()).toList();
  List<List<int>> toInt() => map((e) => e.toInt()).toList();
}
